
# Functions for DPP4 Draft decision, data preparation, and model comparison
# May 2024

# Data read, extract, transform ----
read_bind_ID_data <- function(){
  
  # Read in and combine ID data
  # These data files are from Commission website, downloaded and saved to Inputs 
  # https://comcom.govt.nz/__data/assets/file/0018/302436/Electricity-distributors-information-disclosure-data-2019-2023.xlsx
  # https://comcom.govt.nz/__data/assets/file/0035/337958/Electricity-distributors-information-disclosure-data-2013-2018.xlsx
  
  file_ID_2013_18 <- 
    file.path("Inputs","Electricity-distributors-information-disclosure-data-2013-2018.xlsx")
  
  file_ID_2019_23 <- 
    file.path("Inputs","Electricity-distributors-information-disclosure-data-2019-2023.xlsx")
  
  d_1318 <- read_xlsx(file_ID_2013_18, sheet = "Database", skip = 0)
  d_1923 <- read_xlsx(file_ID_2019_23, sheet = "Database", skip = 0)
  
  stopifnot(sum(names(d_1318)!=names(d_1923))==0)
  dt <- data.table(rbind(d_1318, d_1923))
  return(dt)
}

extract_transform_ID_data <- function(dt){
  
  # Data prep to extract cost and driver data for analysis 
  
  # Tidy up names
  setnames(dt, 
           old = c("EDB", 
                   "Disclosure year",
                   "Observation year",
                   "Forecast year",
                   "Sub-category"
           ), 
           new = c("edb",
                   "disclosure_year",
                   "observation_year",
                   "forecast_year",
                   "sub_category")
  )
  setnames(dt, old = names(dt), new = tolower(names(dt)))
  
  # Set up Structure to pull out variables of interest
  # Will then later filter to those obs. and widen for excel output
  dt <- dt[, -"text_input"]
  dt[, year := ""]
  dt[, var := ""]
  dt[, var_value := ""]
  setcolorder(dt, c("edb", "year", "var", "var_value"))
  
  # 15 March 2024 - how to treat Lakeland in  OtagoNet reporting. 
  # LNL is an embedded network, within Aurora's Queenstown network. 
  # Data here for OtagoNet split-out some Lakeland numbers 
  # (Network = Lakeland Frankton or Otago) from 2020, but only the splits required
  # for reporting sub-networks. So we have separate ICP and lines numbers, for
  # example, but not separate Opex numbers. Prior to 2020, all Lakeland numbers 
  # seem to have been included in the overall numbers reported by OtagoNet. 
  # This can be seen by inspection of ICP and lines numbers.
  
  dt_Lakeland <- dt[(edb=="OtagoNet" & network == "Lakeland Frankton"),]
  dt_Lakeland[, edb := "Lakeland"]
  
  # For all others, retain only total of subnetworks 
  dt <- dt[network =="All",]
  
  table(dt$edb, dt$year)
  
  # Change edb name for historic data from Eastland to FirstLight
  # In data read-in, only 2023 has FirstLight, but change past to match present
  dt[edb=="Eastland Network", edb := "Firstlight Network"]
  
  # Read data from ID data sets, using rules determined by DP via inspection
  
  # icp
  # Total across sub-networks
  dt[(section == "8(i): Billed Quantities by Price Component" &
        category == "Total" &
        sub_category == "Consumer totals" &
        description=="Average no. of ICPs in disclosure year" &
        disclosure_year == observation_year),
     `:=`(year=observation_year,
          var = "icp",
          var_value = value)]
  
  # lines
  # Total overground and underground, across sub-networks
  dt[(section == "9c: Overhead lines and underground cables" &
        category == "Total circuit length (for supply)" &
        sub_category == "Total circuit length (km)" &
        description=="Total circuit length (for supply)" &
        disclosure_year == observation_year),
     `:=`(year=observation_year,
          var = "lines",
          var_value = value)]
  
  # peak
  # Maximum coincident system demand [MW] across sub-networks (non additive)
  dt[(section == "9e(ii): System Demand" &
        category == "Maximum coincident system demand" &
        #sub_category == "" & # blank
        description=="Maximum coincident system demand" &
        disclosure_year == observation_year),
     `:=`(year = observation_year,
          var = "peak",
          var_value = value)]
  
  # delivery
  # Total energy [MWh] delivered in the year
  dt[(section == "9e(ii): System Demand" &
        category == "Electricity volumes carried" &
        #sub_category == "" & #Blank field
        description=="Total energy delivered to ICPs" &
        disclosure_year == observation_year),
     `:=`(year = observation_year,
          var = "delivery",
          var_value = value)]
  
  # ExpenditureOnAssets
  # (capex)
  dt[(section == "6a(i): Expenditure on Assets" &
        category == "Expenditure on assets" &
        sub_category == "Expenditure on assets" & 
        description =="Expenditure on assets" &
        disclosure_year == observation_year),
     `:=`(year = observation_year,
          var = "ExpenditureOnAssets",
          var_value = value)]
  
  # network
  # Network Opex
  dt[(section == "6b(i): Operational Expenditure" &
        category == "Network opex" &
        #sub_category == "" &  # blank 
        description =="Network opex" &
        disclosure_year == observation_year),
     `:=`(year = observation_year,
          var = "network",
          var_value = value)]
  
  # no-arr
  # Network Opex, Asset replacement and renewal
  dt[(section == "6b(i): Operational Expenditure" &
        category == "Network opex" &
        #sub_category == "" &  # blank 
        description =="Asset replacement and renewal" &
        disclosure_year == observation_year),
     `:=`(year = observation_year,
          var = "no-arr",
          var_value = value)]
  
  # no-vm
  # Network Opex, Vegetation management
  dt[(section == "6b(i): Operational Expenditure" &
        category == "Network opex" &
        #sub_category == "" &  # blank 
        description =="Vegetation management" &
        disclosure_year == observation_year),
     `:=`(year = observation_year,
          var = "no-vm",
          var_value = value)]
  
  # no-sie
  # Network Opex, Asset replacement and renewal
  dt[(section == "6b(i): Operational Expenditure" &
        category == "Network opex" &
        #sub_category == "" &  # blank 
        description =="Service interruptions and emergencies" &
        disclosure_year == observation_year),
     `:=`(year = observation_year,
          var = "no-sie",
          var_value = value)]
  
  # no-rcm
  # Network Opex, Routine and corrective maintenance and inspection
  dt[(section == "6b(i): Operational Expenditure" &
        category == "Network opex" &
        #sub_category == "" &  # blank 
        description =="Routine and corrective maintenance and inspection" &
        disclosure_year == observation_year),
     `:=`(year = observation_year,
          var = "no-rcm",
          var_value = value)]
  
  
  # nonnetwork
  # Non-network Opex
  dt[(section == "6b(i): Operational Expenditure" &
        category == "Non-network opex" &
        #sub_category == "" &  # blank 
        description =="Non-network opex" &
        disclosure_year == observation_year),
     `:=`(year = observation_year,
          var = "nonnetwork",
          var_value = value)]
  
  # nno-bs
  # Non-network Opex, Business support
  dt[(section == "6b(i): Operational Expenditure" &
        category == "Non-network opex" &
        #sub_category == "" &  # blank 
        description =="Business support" &
        disclosure_year == observation_year),
     `:=`(year = observation_year,
          var = "nno-bs",
          var_value = value)]
  
  # nno-sons
  # Non-network Opex, System operations and network support
  dt[(section == "6b(i): Operational Expenditure" &
        category == "Non-network opex" &
        #sub_category == "" &  # blank 
        description =="System operations and network support" &
        disclosure_year == observation_year),
     `:=`(year = observation_year,
          var = "nno-sons",
          var_value = value)]
  
  # insurance
  # Insurance is a part of non-network Opex, System operations and network support
  dt[(section == "6b(ii): Subcomponents of Operational Expenditure (where known)" &
        category == "Expenditure on subcomponents" &
        #sub_category == "" &  # blank 
        description =="Insurance" &
        disclosure_year == observation_year),
     `:=`(year = observation_year,
          var = "ins_actual",
          var_value = value)]
  
  # Prelim. Data Checks
  # counts <- t(table(dt$var, dt$edb))
  # counts
  # Alpine Energy 2014 icp count missing (icp) 
  # 8(i): Billed Quantities by Price Component 
  
  # Aurora 2023 Asset replacement and renewal missing (no-arr)
  # 6b(i): Operational Expenditure, Description 'Asset replacement and renewal'
  
  # Also see more below for no-arr and no-vm missing data
  
  # Retain only data we want
  
  names(data)
  cols_keep <- c("edb", "year", "var", "var_value","units")
  data <- dt[get("year")>2000, ..cols_keep]
  
  # Create wide format table for export as csv
  cols_order <- c("edb", "year", "icp", "lines", "peak", "delivery", 
                  "ExpenditureOnAssets", "network", "nonnetwork", 
                  "no-arr", "no-rcm", "no-sie", "no-vm", "nno-bs", "nno-sons",
                  "ins_actual")
  
  data_wide <- dcast(data, edb + year ~ var, value.var = "var_value")
  setcolorder(data_wide, cols_order)
  
  # Join exempt status from 2022 (does not change over time in this data)
  
  exempt_status <- data.table(read.csv("Inputs/EDB_exempt_table.csv"))
  exempt_status <- exempt_status[year==2022,]
  exempt_status$year <- NULL
  
  data_out <- merge(x = data_wide, y = exempt_status, by = "edb", all.x = TRUE)
  data_out[edb == "Firstlight Network", exempt := 0]
  
  # Data checks
  
  # Spot checks for Alpine, Aurora and Vector, from previous Opex data v0.2 and data_out confirms:
  # - MCSD peak values same for Alpine and halved for Aurora and Vector, correcting prior double counting
  # - same values for other variables
  # - some data points missing from ID data
  #
  # As below, the missing data are:
  # data_out[!complete.cases(data_out),]
  #    + icp    / Alpine 2014
  #    + no-arr / Aurora 2023; Mainpower 2017, 2018
  #    + no-vm  / Centralines 2013, Electricity Invercargill 2022, 
  #               The Power Company 2013, Wellington Electricity 2013, 
  #               Westpower 2013
  #
  # To the extent that fitting will be done for 2018 forwards (ie 5 years forward 
  # from DPP3), only on exempt edbs, and not using dis-aggregated costs,
  # none of these are major issues
  
  # We manually add the 2014 icp number for Alpine, from previous files
  data_out[edb=="Alpine Energy" & year == 2014, icp := 31413]
  
  file_write <- file.path("Inputs", "DPP4_opex_data_from_ID.csv")
  write.csv(data_out, file = file_write, row.names = FALSE)
  
  return (data_out)
  
}


prepare_opex_data <- function(dt_in){
  
  dt = copy(data.table(dt_in))
  
  # Tidy up data file
  dt <- data.table(dt)
  names(dt) <- tolower(names(dt))
  names(dt)
  
  dt[ ,`:=`(icp = as.numeric(icp),
            lines = as.numeric(lines), 
            year = as.numeric(year))]
  
  # Adjust for operating leases using DPP3 publisehd data file
  oplease_file = file.path("Inputs","Operating-lease-model-EDB-DPP3-final-determination-27-November-2019.xlsx")
  
  # Read in operating lease data and apply adjustments per CEPA 
  #oplease.file <- file.path("Inputs", "Operating-lease-model-EDB-DPP3-final-determination-27-November-2019.xlsx")
  
  OpLease <- read_xlsx(oplease_file, sheet = "Output (opex proj)", skip = 5)
  OpLease <- filter(OpLease, !is.na(Name))
  colnames(OpLease)[2:3] <- c("Year","Type")
  OpLease <- filter(OpLease, !is.na(Year))
  OpLease <- mutate(OpLease, Name = ifelse(grepl("Non-network", Name, fixed = TRUE),"nonnetworkAdj","networkAdj"))
  OpLease <- dplyr::select(OpLease, 1:2,4:20)
  OpLease <- pivot_longer(OpLease,3:19)
  OpLease <- OpLease %>% group_by(Name,Year,name) %>% summarise(Value = sum(value))
  OpLease_nonNetwork <- filter(OpLease, Name == "nonnetworkAdj") %>% 
    ungroup %>% dplyr::select(year = Year, edb = name, nonnetworkAdj = Value)
  OpLease_Network <- filter(OpLease, Name == "networkAdj")  %>% 
    ungroup %>% dplyr::select(year = Year, edb = name, networkAdj = Value)
  
  # Join Op Lease data
  dt <- left_join(dt, OpLease_nonNetwork, by = c("edb","year"))
  dt <- left_join(dt, OpLease_Network, by = c("edb","year"))
  dt[is.na(dt)] <- 0
  
  glimpse(dt)
  
  dt <- dt %>% mutate(nonnetwork_raw = nonnetwork,
                      network_raw = network)
  
  dt <- dt %>% mutate(nonnetwork = nonnetwork_raw + nonnetworkAdj, 
                          network = network_raw + networkAdj)
  
  # Inflation series DPP3 approach
  #  - Index is (60/40 LCI/PPI) for opex
  #  - index_capex is CGPI index for capex
  index.file <- file.path("Inputs", "Inflation_indices_2013_2023.xlsx")
  InflationIndex <- data.table(read_xlsx(index.file, sheet = "InflationIndex_with_capex"))
  InflationIndex
  # Join Index data and reference to 2019$ (Indexes for 2019 are 1.000)
  dt <- left_join(dt, InflationIndex, by = 'year')
  
  dt <- copy(data.table(dt)) # (Avoids shallow copy issue)
  
  dt[ , `:=`(network_nom = network,
             nonnetwork_nom = nonnetwork,
             capex_nom = expenditureonassets)]
  
  dt[ , `:=`(network = network_nom/ index_opex,
             nonnetwork = nonnetwork_nom/ index_opex,
             capex = capex_nom/index_capex)]
  
  # Ratcheted peak and delivery
  dt <- copy(data.table(dt)) # (Avoids shallow copy issue)
  dt[, peak_rat := cummax(peak),  by = edb]
  dt[, delivery_rat := cummax(delivery), by = edb]
  
  # Take logarithms for model fitting to return elasticities
  # Model variables are all ln transformed. Not renamed purely for convenience
  log_vars <- c("icp", "lines", "network", "nonnetwork", "capex", "peak_rat", "delivery_rat")
  for (var in log_vars) dt[[var]] <- log(dt[[var]])
  
  # Data Outliers - data tuples with with (or both) irregular icp or lines values
  # Rather than adjust (as done for the ICP and lines trend forecasting where
  # there are only 5 data points for EDB) we simply exclude these here. 
  
  irregular_data <- 
    data.table(tibble::tribble(~edb, ~year, 
                               "Alpine Energy", 2022,
                               "Alpine Energy", 2023,
                               "Aurora Energy",  2018,
                               "Aurora Energy",  2019,
                               "Buller Electricity", 2018,
                               "EA Networks", 2018,
                               "Electricity Invercargill",2021,
                               "Firstlight Network", 2021,
                               "Horizon Energy", 2018,
                               "Horizon Energy", 2022,
                               "Horizon Energy", 2023,
                               "Nelson Electricity", 2022,
                               "Nelson Electricity", 2023,
                               "Network Waitaki", 2020,
                               "Network Waitaki", 2021,
                               "OtagoNet", 2023,
                               "Powerco", 2021,
                               "Scanpower", 2021,
                               "The Lines Company", 2018,
                               "The Lines Company", 2019,
                               "The Power Company", 2020,
                               "Top Energy", 2023,
                               "Unison Networks", 2023,
                               "Westpower", 2022,
                               "Westpower", 2023
    )
    )
  
  irregular_data[ ,"outlier_flag" := 1]
  
  # Check that all names above are correct 
  stopifnot(sum(!irregular_data$edb %in% unique(dt$edb))==0)
  
  # Join and set zero any missing values
  data_outliers <- data.table(merge(x = dt, 
                                    y = irregular_data, 
                                    by = c("edb", "year"), 
                                    all.x = TRUE))
  data <- copy(data_outliers[is.na(outlier_flag), "outlier_flag" := 0])
  
  return(data)
  
}

full_data_preparation <- function(){
  
  reload_data = TRUE
  
  # Takes  ~ 15 seconds to reload 2 large excel files from 'Inputs' directory 
  # and extract data. Ignore warnings related to 'Expecting logical'
  # If output not there, force data preparation
  
  is_no_data_file <- !file.exists(file.path("Inputs", "DPP4_opex_data_from_ID.csv"))
  is_no_data_file
  
  if (reload_data | is_no_data_file)   {
    
    id_data <- read_bind_ID_data()
    data_read <- extract_transform_ID_data(id_data)
  
    }else {
  
    data_read <- data.table(read.csv(file.path("Inputs", "DPP4_opex_data_from_ID.csv"),
                                     header = TRUE, stringsAsFactors = FALSE))
  }
  
  # Ensure all fields numeric, except edb
  make_num <- names(data_read)[!names(data_read)%in%c("edb", "year", "exempt")]
  make_int <- c("year", "exempt")
  data_read[, (make_num) := lapply(.SD, as.numeric), .SDcols = make_num]
  data_read[, (make_int) := lapply(.SD, as.integer), .SDcols = make_int]
  
  # Prepare data, including adjust for operating leases, deflate to constant dollar basis
  # log transform key modelling quantities
  data_to_model <- prepare_opex_data(data_read)
  
  write.csv(file = file.path("Inputs", "DPP4_opex_data_to_model.csv"), data_to_model)
  
  return(data_to_model)
}


# Record Multiple Model Outputs Function ----

# For collating model outputs to write out as csv 
Output <- data.frame(matrix(nrow = 0, ncol = 0)) 

Record <- function(Model){
  
  if(class(Model) == "lm"){
    
    ModelOutputs <- Model$coefficients %>% as.data.frame()
    AdjRSquared <- summary(Model)$adj.r.squared
    
    colnames(ModelOutputs) <- "Data"
    ModelOutputs$Name <- row.names(ModelOutputs)
    
    ModelOutputs <- pivot_wider(ModelOutputs, values_from = "Data", names_from = "Name")
    
    StdErrs <- summary(Model)$coefficients[,2] %>% as.data.frame()
    StdErrs$Name <- row.names(StdErrs)
    StdErrs <- pivot_wider(StdErrs, values_from = ".", names_from = "Name")
    colnames(StdErrs) <- paste0("StdErrs-", colnames(StdErrs))
    
    PValues <- summary(Model)$coefficients[,4] %>% as.data.frame()
    PValues$Name <- row.names(PValues)
    PValues <- pivot_wider(PValues, values_from = ".", names_from = "Name")
    colnames(PValues) <- paste0("PValue-", colnames(PValues))
    
    ModelOutputs <- bind_cols(ModelOutputs, StdErrs, PValues)
    
    ModelOutputs$ModelCount <- nrow(Output) + 1
    ModelOutputs$ModelName <- deparse(substitute(Model))
    ModelOutputs$AdjRSquared  <- summary(Model)$adj.r.squared
    ModelOutputs$NetworkCost <- names(model.frame(Model))[1]
    ModelOutputs$Type <- "Not Robust"

    Output <<- bind_rows(Output, ModelOutputs)
    
  }
  
  if(class(Model) == "lm_robust"){
    
    ModelOutputs <- Model$coefficients %>% as.data.frame()
    AdjRSquared <- summary(Model)$adj.r.squared
    
    colnames(ModelOutputs) <- "Data"
    ModelOutputs$Name <- row.names(ModelOutputs)
    
    ModelOutputs <- pivot_wider(ModelOutputs, values_from = "Data", names_from = "Name")
    
    PValues <- Model$p.value %>% as.data.frame()
    PValues$Name <- row.names(PValues)
    PValues <- pivot_wider(PValues, values_from = ".", names_from = "Name")
    colnames(PValues) <- paste0("PValue-", colnames(PValues))
    
    ModelOutputs <- bind_cols(ModelOutputs, PValues)
    
    # Have not include stderrs here yet (As not using rn)
    
    ModelOutputs$ModelCount <- nrow(Output) + 1
    ModelOutputs$ModelName <- deparse(substitute(Model))
    ModelOutputs$AdjRSquared  <- summary(Model)$adj.r.squared
    ModelOutputs$NetworkCost <- names(model.frame(Model))[1]
    ModelOutputs$Type <- "Robust"
 
    Output <<- bind_rows(Output, ModelOutputs)
    
  }
  
}

# Test outliers functions ----

outlierN_fail = 3

test_outlier1 <- function(model) {
  
  yhattt <- fitted(model)
  e <- residuals(model)
  lev <- hatvalues(model)
  influence <- influence.measures(model)[1] %>% as.data.frame()
  dfits <- influence$infmat.dffit
  
  #Implemented Welsch's distance following Stata manual pg. 11: 
  # https://www.stata.com/manuals/rregresspostestimation.pdf
  wd <- dfits * sqrt((nobs(model)-1)/(1-lev)) 
  cooksd <- cooks.distance(model)
  
  k <- length(coef(model)) 
  coef(model)
  
  g1 <- as.numeric(abs(dfits) > 2 * sqrt(k / nobs(model)))
  dfits_outlier_stat <- abs(dfits)
  
  g2 <- as.numeric(abs(cooksd) > 4 / nobs(model))
  cooksd_outlier_stat <- abs(cooksd)
  
  g3 <- as.numeric(abs(wd) > 3 * sqrt(k))
  wd_outlier_stat <- abs(wd)
  
  g4 <- as.numeric(lev > (2 * k) / nobs(model))
  lev_outlier_stat <- abs(lev)
  
  outlier_vars <- cbind(g1, g2, g3, g4)
  outlier_vars[is.na(outlier_vars)] <- 0
  
  sum_outlier <- rowSums(outlier_vars)
  
  model$model
  outliers <- data.frame(
    row_number = rownames(model$model),
    sum_outlier = sum_outlier
  )
  
  outliers <- filter(outliers, sum_outlier > 1)
  
  return(outliers)
}

test_outlier_loop1 <- function(DataNew, Model){
  
  #Initialise helper columns in DataNew
  DataNew$N1outlier <- 0
  DataNew$sum_outlier <- 0
  
  Looptest <- outlierN_fail
  
  while(Looptest >= outlierN_fail){
    
    DataNew <- dplyr::select(DataNew,-sum_outlier)
    
    model_iter <- DataNew %>% filter(N1outlier == 0) %>% lm(Model, data = .)
    
    #print(model_iter)
    
    DataTemp <- DataNew %>% filter(N1outlier == 0)
    
    DataTemp$row_number <- rownames(DataTemp)
    
    Outliers <- test_outlier1(model_iter)
    #print(Outliers)
    
    DataTemp <- left_join(DataTemp, Outliers, by = 'row_number')
    
    DataTemp <- dplyr::select(DataTemp,edb,year,sum_outlier)
    
    DataNew <- left_join(DataNew, DataTemp, by = c('edb','year'))
    
    DataNew <- DataNew %>% 
      mutate(N1outlier = ifelse(sum_outlier>= outlierN_fail | N1outlier== 1,1,0))  %>% 
      mutate(N1outlier = ifelse(is.na(N1outlier),0,N1outlier))
    
    Looptest <- max(Outliers$sum_outlier)
    outputs <- list("model_iter" = model_iter,
                    "data_fit" = DataNew)
  }
  return(outputs)
}


test_outlier_loop_robust1 <- function(DataNew, Model){
  
  DataNew$N1outlier <- 0
  DataNew$sum_outlier <- 0
  
  Looptest <- outlierN_fail
  
  while(Looptest >= outlierN_fail){
    
    DataNew <- dplyr::select(DataNew,-sum_outlier)
    
    model_iter <- DataNew %>% filter(N1outlier == 0) %>% lm(Model, data = .)
    
    #print(model_iter)
    
    DataTemp <- DataNew %>% filter(N1outlier == 0)
    
    DataTemp$row_number <- rownames(DataTemp)
    
    Outliers <- test_outlier1(model_iter)
    
    DataTemp <- left_join(DataTemp, Outliers, by = 'row_number')
    
    DataTemp <- dplyr::select(DataTemp,edb,year,sum_outlier)
    
    DataNew <- left_join(DataNew, DataTemp, by = c('edb','year'))
    
    DataNew <- DataNew %>%
      mutate(N1outlier = ifelse(sum_outlier>= outlierN_fail | N1outlier== 1,1,0))  %>% 
      mutate(N1outlier = ifelse(is.na(N1outlier),0,N1outlier))
    
    #Outliers
    
    Looptest <- max(Outliers$sum_outlier)
    
    model_Robust <- DataNew %>% 
      filter(N1outlier == 0) %>% 
      lm_robust(as.formula(Model), data = ., clusters = edb, se_type = "stata")
    
    outputs <- list("model_iter_robust" = model_Robust,
                    "data_fit" = DataNew)
  }
  return(outputs)
}

# Named lists for stargazer tables ----

makeNamedList <- function(...) {
  structure(list(...), names = as.list(substitute(list(...)))[-1L])
}


# Add dependent-variable names for modelsummary ----

model_summary_insert_dvnames <- function(x) {
  out <- c("Opex category", names(dvnames(x)))
  out <- data.frame(as.list(out))
  attr(out, "position") <- 0
  return(out)
}

