
# 21 May 2024
# File prepared for DPP4 Draft Decisions 

# Set Up -----------------------------------------------------------------------

rm(list=ls())
library(here)
source(file.path(here(),'context.R'))
source(file.path(here(),'R','dpp4_dd_model_fit_data_functions.R'))

# Data Parameters --------------------------------------------------------------

year_min_fit = 2018
year_max_fit = 2023
leave_in_irregular_ID_data = FALSE # False to omit irregular data

# First time, set prepare_data_again = TRUE

# After this, re-prepare data if necessary, or read in data file below
# Takes ~ 15 seconds to extract data from large EXCEL files

prepare_data_again = TRUE  # TRUE to prepare, save DPP4_opex_data_to_model.csv
#prepare_data_again = FALSE  # FALSE to use existing DPP4_opex_data_to_model.csv   

# Prepare or load data----------------------------------------------------------

if(prepare_data_again) data <- full_data_preparation()
  
# Read data
stopifnot(file.exists(file.path("Inputs", "DPP4_opex_data_to_model.csv")))
data <- data.table(read.csv(file.path("Inputs", "DPP4_opex_data_to_model.csv"), 
                            stringsAsFactors = FALSE))

# Option to leave in (not exclude) irregular input outlier_flag = 1 in data prep
if (leave_in_irregular_ID_data) data[ ,outlier_flag := 0]

# Model Definition -------------------------------------------------------------

# Dimensions to consider model fitting
# 1. Date Range - all data is 2013-2022; draft DPP4 reference period 2018-2023 
# 2. Base model - and assess substitution and additions to ICP + lines
# 3. Inclusion of a capex term?
# 4. Inclusion of a time term?
# 5. Outliers: iterative model outlier exclusion 

# Uses custom 'Record' function to collate model outputs, written to Outputs/

# First clear collated results.
refresh_output = TRUE
if (refresh_output) Output <- data.frame(matrix(nrow = 0, ncol = 0)) 

# Data range and outliers
# outlier_flag==0 are data tuples (edb-year) NOT identified as irregular
data_all <- data
data_ref <- data[year >= year_min_fit & year <= year_max_fit & outlier_flag==0]

# 1. Network opex models ----

# Preferred Model = icp + lines, since 2018
modelN_lines_icp <- lm(network ~ lines + icp, data = data_ref)
Record(modelN_lines_icp)

# All years, no outlier exclusion
modelN_lines_icp_2013_2023 <- lm(network ~ lines + icp, data = data_all[outlier_flag==0])
Record(modelN_lines_icp_2013_2023)

# Since 2018, no outlier exclusion
modelN_lines_icp_2018_2023_all <- lm(network ~ lines + icp, data = data_all[year >= 2018])
Record(modelN_lines_icp_2018_2023_all)

# Preferred w capex
modelN_lines_icp_capex <- lm(network ~ lines + icp + capex, data = data_ref)
Record(modelN_lines_icp_capex)

# Preferred w time
modelN_lines_icp_time <- lm(network ~ lines + icp + year, data = data_ref)
Record(modelN_lines_icp_time)

# Preferred w capex and time
modelN_lines_icp_capex_time <- lm(network ~ lines + icp + capex + year, data = data_ref)
Record(modelN_lines_icp_capex_time)

# Pref w peak_ratchet
modelN_lines_icp_peak_rat <- lm(network ~ lines + icp + peak_rat, data = data_ref)
Record(modelN_lines_icp_peak_rat)

# Pref w delivery_ratchet (total energy)
modelN_lines_icp_del_rat <- lm(network ~ lines + icp + delivery_rat, data = data_ref) 
Record(modelN_lines_icp_del_rat)

# Pref w delivery (total energy)
modelN_lines_icp_delivery <- lm(network ~ lines + icp + delivery, data = data_ref) 
Record(modelN_lines_icp_delivery)

# Substitutions

# lines + peak_rat
modelN_lines_peak_rat <- lm(network ~ lines + peak_rat, data = data_ref)
Record(modelN_lines_peak_rat)

# lines + delivery_rat
modelN_lines_delivery_rat <- lm(network ~ lines + delivery_rat, data = data_ref)
Record(modelN_lines_delivery_rat)

# lines + delivery
modelN_lines_delivery <- lm(network ~ lines + delivery, data = data_ref)
Record(modelN_lines_delivery)

# icp + peak_rat
modelN_icp_peak_rat <- lm(network ~ icp + peak_rat, data = data_ref)
Record(modelN_icp_peak_rat)

# icp + delivery_rat
modelN_icp_delivery_rat <- lm(network ~ icp + delivery_rat, data = data_ref)
Record(modelN_icp_delivery_rat)

# icp + delivery
modelN_icp_delivery <- lm(network ~ icp + delivery, data = data_ref)
Record(modelN_icp_delivery)

# Additions

# lines + icp + peak_rat
modelN_lines_icp_peak_rat <- lm(network ~ lines + icp + peak_rat, data = data_ref)
Record(modelN_lines_icp_peak_rat)

# lines + icp + delivery_rat
modelN_lines_icp_delivery_rat <- lm(network ~ lines + icp + delivery_rat, data = data_ref)
Record(modelN_lines_icp_delivery_rat)

# lines + icp + delivery
modelN_lines_icp_delivery <- lm(network ~ lines + icp + delivery, data = data_ref)
Record(modelN_lines_icp_delivery)

# 2.Non-network opex models ----

# Preferred Model = icp + lines, since 2018
modelNN_lines_icp <- lm(nonnetwork ~ lines + icp, data = data_ref)
Record(modelNN_lines_icp)

# All years, no outlier exclusion
modelNN_lines_icp_2013_2023 <- lm(nonnetwork ~ lines + icp, data = data_all[outlier_flag==0])
Record(modelNN_lines_icp_2013_2023)

# Since 2018, no outlier exclusion
modelNN_lines_icp_2018_2023_all <- lm(nonnetwork ~ lines + icp, data = data_all[year >= 2018])
Record(modelNN_lines_icp_2018_2023_all)

# Preferred w capex
modelNN_lines_icp_capex <- lm(nonnetwork ~ lines + icp + capex, data = data_ref)
Record(modelNN_lines_icp_capex)

# Preferred w time
modelNN_lines_icp_time <- lm(nonnetwork ~ lines + icp + year, data = data_ref)
Record(modelNN_lines_icp_time)

# Preferred w capex and time
modelNN_lines_icp_capex_time <- lm(nonnetwork ~ lines + icp + capex + year, data = data_ref)
Record(modelNN_lines_icp_capex_time)

# Pref w peak_ratchet
modelNN_lines_icp_peak_rat <- lm(nonnetwork ~ lines + icp + peak_rat, data = data_ref)
Record(modelNN_lines_icp_peak_rat)

# Pref w delivery_ratchet (total energy)
modelNN_lines_icp_delivery_rat <- lm(nonnetwork ~ lines + icp + delivery_rat, data = data_ref)
Record(modelNN_lines_icp_delivery_rat)

# Pref w delivery (total energy)
modelNN_lines_icp_delivery <- lm(nonnetwork ~ lines + icp + delivery, data = data_ref)
Record(modelNN_lines_icp_delivery)

# SUBSTITUTIONS

# lines + peak_rat
modelNN_lines_peak_rat <- lm(nonnetwork ~ lines + peak_rat, data = data_ref)
Record(modelNN_lines_peak_rat)

# lines + delivery_rat
modelNN_lines_delivery_rat <- lm(nonnetwork ~ lines + delivery_rat, data = data_ref)
Record(modelNN_lines_delivery_rat)

# lines + delivery
modelNN_lines_delivery <- lm(nonnetwork ~ lines + delivery, data = data_ref)
Record(modelNN_lines_delivery)

# lines + peak_rat
modelNN_lines_peak_rat <- lm(nonnetwork ~ lines + peak_rat, data = data_ref)
Record(modelNN_lines_peak_rat)

# lines + delivery_rat
modelNN_lines_delivery_rat <- lm(nonnetwork ~ lines + delivery_rat, data = data_ref)
Record(modelNN_lines_delivery_rat)

# lines + delivery
modelNN_lines_delivery <- lm(nonnetwork ~ lines + delivery, data = data_ref)
Record(modelNN_lines_delivery)

# icp + peak_rat
modelNN_icp_peak_rat <- lm(nonnetwork ~ icp + peak_rat, data = data_ref)
Record(modelNN_icp_peak_rat)

# icp + delivery_rat
modelNN_icp_delivery_rat <- lm(nonnetwork ~ icp + delivery_rat, data = data_ref)
Record(modelNN_icp_delivery_rat)

# icp + delivery
modelNN_icp_delivery <- lm(nonnetwork ~ icp + delivery, data = data_ref)
Record(modelNN_icp_delivery)

# ADDITIONS
# lines + icp + peak_rat
modelNN_lines_icp_peak_rat <- lm(nonnetwork ~ lines + icp + peak_rat, data = data_ref)
Record(modelNN_lines_icp_peak_rat)

# lines + icp + delivery_rat
modelNN_lines_icp_delivery_rat <- lm(nonnetwork ~ lines + icp + delivery_rat, data = data_ref)
Record(modelNN_lines_icp_delivery_rat)

# lines + icp + delivery
modelNN_lines_icp_delivery <- lm(nonnetwork ~ lines + icp + delivery, data = data_ref)
Record(modelNN_lines_icp_delivery)

# 3. Output stargazer tables  ----

# Add AIC and BIC to lm() models 
# Laborious but transparent

# Network
modelN_lines_icp$AIC <- AIC(modelN_lines_icp)
modelN_lines_icp$BIC <- BIC(modelN_lines_icp)

modelN_lines_icp_2013_2023$AIC <- AIC(modelN_lines_icp_2013_2023)
modelN_lines_icp_2013_2023$BIC <- BIC(modelN_lines_icp_2013_2023)
modelN_lines_icp_2018_2023_all$AIC <- AIC(modelN_lines_icp_2018_2023_all)
modelN_lines_icp_2018_2023_all$BIC <- BIC(modelN_lines_icp_2018_2023_all)

modelN_lines_icp_capex$AIC <- AIC(modelN_lines_icp_capex)
modelN_lines_icp_capex$BIC <- BIC(modelN_lines_icp_capex)
modelN_lines_icp_time$AIC <- AIC(modelN_lines_icp_time)
modelN_lines_icp_time$BIC <- BIC(modelN_lines_icp_time)
modelN_lines_icp_capex_time$AIC <- AIC(modelN_lines_icp_capex_time)
modelN_lines_icp_capex_time$BIC <- BIC(modelN_lines_icp_capex_time)

modelN_icp_peak_rat$AIC <- AIC(modelN_icp_peak_rat)
modelN_icp_peak_rat$BIC <- BIC(modelN_icp_peak_rat)
modelN_icp_delivery_rat$AIC <- AIC(modelN_icp_delivery_rat)
modelN_icp_delivery_rat$BIC <- BIC(modelN_icp_delivery_rat)
modelN_icp_delivery$AIC <- AIC(modelN_icp_delivery)
modelN_icp_delivery$BIC <- BIC(modelN_icp_delivery)

modelN_lines_peak_rat$AIC <- AIC(modelN_lines_peak_rat)
modelN_lines_peak_rat$BIC <- BIC(modelN_lines_peak_rat)
modelN_lines_delivery_rat$AIC <- AIC(modelN_lines_delivery_rat)
modelN_lines_delivery_rat$BIC <- BIC(modelN_lines_delivery_rat)
modelN_lines_delivery$AIC <- AIC(modelN_lines_delivery)
modelN_lines_delivery$BIC <- BIC(modelN_lines_delivery)

modelN_lines_icp_peak_rat$AIC <- AIC(modelN_lines_icp_peak_rat)
modelN_lines_icp_peak_rat$BIC <- BIC(modelN_lines_icp_peak_rat)
modelN_lines_icp_delivery_rat$AIC <- AIC(modelN_lines_icp_delivery_rat)
modelN_lines_icp_delivery_rat$BIC <- BIC(modelN_lines_icp_delivery_rat)
modelN_lines_icp_delivery$AIC <- AIC(modelN_lines_icp_delivery)
modelN_lines_icp_delivery$BIC <- BIC(modelN_lines_icp_delivery)

# Non-network
modelNN_lines_icp$AIC <- AIC(modelNN_lines_icp)
modelNN_lines_icp$BIC <- BIC(modelNN_lines_icp)

modelNN_lines_icp_2013_2023$AIC <- AIC(modelNN_lines_icp_2013_2023)
modelNN_lines_icp_2013_2023$BIC <- BIC(modelNN_lines_icp_2013_2023)
modelNN_lines_icp_2018_2023_all$AIC <- AIC(modelNN_lines_icp_2018_2023_all)
modelNN_lines_icp_2018_2023_all$BIC <- BIC(modelNN_lines_icp_2018_2023_all)

modelNN_lines_icp_capex$AIC <- AIC(modelNN_lines_icp_capex)
modelNN_lines_icp_capex$BIC <- BIC(modelNN_lines_icp_capex)
modelNN_lines_icp_time$AIC <- AIC(modelNN_lines_icp_time)
modelNN_lines_icp_time$BIC <- BIC(modelNN_lines_icp_time)
modelNN_lines_icp_capex_time$AIC <- AIC(modelNN_lines_icp_capex_time)
modelNN_lines_icp_capex_time$BIC <- BIC(modelNN_lines_icp_capex_time)

modelNN_icp_peak_rat$AIC <- AIC(modelNN_icp_peak_rat)
modelNN_icp_peak_rat$BIC <- BIC(modelNN_icp_peak_rat)
modelNN_icp_delivery_rat$AIC <- AIC(modelNN_icp_delivery_rat)
modelNN_icp_delivery_rat$BIC <- BIC(modelNN_icp_delivery_rat)
modelNN_icp_delivery$AIC <- AIC(modelNN_icp_delivery)
modelNN_icp_delivery$BIC <- BIC(modelNN_icp_delivery)

modelNN_lines_peak_rat$AIC <- AIC(modelNN_lines_peak_rat)
modelNN_lines_peak_rat$BIC <- BIC(modelNN_lines_peak_rat)
modelNN_lines_delivery_rat$AIC <- AIC(modelNN_lines_delivery_rat)
modelNN_lines_delivery_rat$BIC <- BIC(modelNN_lines_delivery_rat)
modelNN_lines_delivery$AIC <- AIC(modelNN_lines_delivery)
modelNN_lines_delivery$BIC <- BIC(modelNN_lines_delivery)

modelNN_lines_icp_peak_rat$AIC <- AIC(modelNN_lines_icp_peak_rat)
modelNN_lines_icp_peak_rat$BIC <- BIC(modelNN_lines_icp_peak_rat)
modelNN_lines_icp_delivery_rat$AIC <- AIC(modelNN_lines_icp_delivery_rat)
modelNN_lines_icp_delivery_rat$BIC <- BIC(modelNN_lines_icp_delivery_rat)
modelNN_lines_icp_delivery$AIC <- AIC(modelNN_lines_icp_delivery)
modelNN_lines_icp_delivery$BIC <- BIC(modelNN_lines_icp_delivery)

# Output the collated model file

date = format(now(), "%Y%m%d")
file_name <- file.path("Outputs", paste0("model_outputs_",date, ".csv"))
write.csv(Output, file_name, row.names = FALSE)

# Set up named lists required for stargazer table inputs

named_models <- makeNamedList(modelN_lines_icp,
                              modelN_lines_icp_2018_2023_all,
                              modelN_lines_icp_2013_2023,
                              modelN_lines_peak_rat, 
                              modelN_lines_delivery_rat, 
                              modelN_lines_delivery, 
                              modelN_icp_peak_rat, 
                              modelN_icp_delivery_rat, 
                              modelN_icp_delivery, 
                              modelN_lines_icp_peak_rat,  
                              modelN_lines_icp_delivery_rat,
                              modelN_lines_icp_delivery,
                              modelN_lines_icp_capex,
                              modelN_lines_icp_time, 
                              modelN_lines_icp_capex_time, 
                              
                              modelNN_lines_icp,
                              modelNN_lines_icp_2018_2023_all,
                              modelNN_lines_icp_2013_2023,
                              modelNN_lines_peak_rat, 
                              modelNN_lines_delivery_rat, 
                              modelNN_lines_delivery, 
                              modelNN_icp_peak_rat, 
                              modelNN_icp_delivery_rat, 
                              modelNN_icp_delivery, 
                              modelNN_lines_icp_peak_rat,  
                              modelNN_lines_icp_delivery_rat,
                              modelNN_lines_icp_delivery,
                              modelNN_lines_icp_capex,
                              modelNN_lines_icp_time, 
                              modelNN_lines_icp_capex_time
                              )
  
models_N_list_base <- list(modelN_lines_icp,
                           modelN_lines_peak_rat, 
                           modelN_lines_delivery_rat, 
                           modelN_lines_delivery, 
                           modelN_icp_peak_rat, 
                           modelN_icp_delivery_rat,
                           modelN_icp_delivery,
                           modelN_lines_icp_peak_rat, 
                           modelN_lines_icp_delivery_rat,
                           modelN_lines_icp_delivery)

col_labels_N_base  = c("lines+icp",  
                       "lines+peakR", 
                       "lines+delR",
                       "lines+del",
                       "icp+peakR", 
                       "icp+delR",
                       "icp+del",
                       "+peakR",
                       "+deliveryR",
                       "+delivery")

models_NN_list_base <- list(modelNN_lines_icp,
                            modelNN_lines_peak_rat, 
                            modelNN_lines_delivery_rat, 
                            modelNN_lines_delivery, 
                            modelNN_icp_peak_rat, 
                            modelNN_icp_delivery_rat,
                            modelNN_icp_delivery,
                            modelNN_lines_icp_peak_rat, 
                            modelNN_lines_icp_delivery_rat,
                            modelNN_lines_icp_delivery)

col_labels_NN_base  = c("lines+icp",  
                   "lines+peakR", 
                   "lines+delR",
                   "lines+del",
                   "icp+peakR", 
                   "icp+delR",
                   "icp+del",
                   "+peakR",
                   "+deliveryR",
                   "+delivery")

       
models_N_list_add <- list(modelN_lines_icp,
                          modelN_lines_icp_capex, 
                          modelN_lines_icp_time, 
                          modelN_lines_icp_capex_time)

col_labels_N_add = c("lines+icp", 
                     "+capex",
                     "+time",
                     "+capex+time")

models_NN_list_add <- list(modelNN_lines_icp,
                           modelNN_lines_icp_capex, 
                           modelNN_lines_icp_time, 
                           modelNN_lines_icp_capex_time)

col_labels_NN_add = c("lines+icp",
                      "+capex",
                      "+time",
                      "+capex+time")
   
# Stack N models using stargazer an doutput to screen
years = paste0(min(data_ref$year), "-", max(data_ref$year))

# a. Comparison table for Network Models - Base ----

stargazer(models_N_list_base,
          title = paste0("Network Models ", years, ". Irregular ID data excluded, without IMOE or CRSE."),
          type = "text",
          column.labels = col_labels_N_base,
          header=F, #  get rid of r package output text
          single.row = F, # put coefficients and standard errors on same line
          no.space = F, # remove the spaces after each line of coefficients
          #column.sep.width = "3pt", # to reduce column width
          font.size = "small",
          # Manually choose here, if standard errors are vanilla, or assume clusters
          # Needs same number of points in each model being compared though
          omit.stat = c("f", "ser"),
          out = file.path("Outputs", "table_sg_ntwk_base.html")
          )

# Model Summary Network models

named_N_models_base <- list(
  "lines+icp" = modelN_lines_icp,
  "lines+peakR" = modelN_lines_peak_rat, 
  "lines+delR" = modelN_lines_delivery_rat, 
  "lines+del" = modelN_lines_delivery, 
  "icp+peakR" = modelN_icp_peak_rat, 
  "icp+delR" = modelN_icp_delivery_rat, 
  "icp+del" = modelN_icp_delivery, 
  "+peakR" = modelN_lines_icp_peak_rat, 
  "+deliveryR" = modelN_lines_icp_delivery_rat,
  "+delivery" = modelN_lines_icp_delivery)
             
modelsummary(named_N_models_base,
             output = file.path("Outputs", "table_ms_network_base.docx"),
             title = paste0('Network models.'),
             gof_omit = "Log|F",
             add_rows = model_summary_insert_dvnames(named_N_models_base),
             stars = TRUE
)


# b. Comparison table for Network Models - Additions ----

stargazer(models_N_list_add,
          title = paste0("Network Model additions ", years, ". Irregular ID data excluded, without IMOE or CRSE."),
          type = "text",
          column.labels = col_labels_N_add,
          header=F, #  get rid of r package output text
          single.row = F, # put coefficients and standard errors on same line
          no.space = F, # remove the spaces after each line of coefficients
          #column.sep.width = "3pt", # to reduce column width
          font.size = "small",
          # Manually choose here, if standard errors are vanilla, or assume clusters
          # Needs same number of points in each model being compared though
          omit.stat = c("f", "ser"),
          out = file.path("Outputs", "table_sg_ntwk_adds.html")
)


# Model Summary Network models

named_N_models_base <- list(
  "lines+icp" = modelN_lines_icp,
  "lines+peakR" = modelN_lines_peak_rat, 
  "lines+delR" = modelN_lines_delivery_rat, 
  "lines+del" = modelN_lines_delivery, 
  "icp+peakR" = modelN_icp_peak_rat, 
  "icp+delR" = modelN_icp_delivery_rat, 
  "icp+del" = modelN_icp_delivery, 
  "+peakR" = modelN_lines_icp_peak_rat, 
  "+deliveryR" = modelN_lines_icp_delivery_rat,
  "+delivery" = modelN_lines_icp_delivery)

modelsummary(named_N_models_base,
             output = file.path("Outputs", "table_ms_network_base.docx"),
             title = paste0('Network models.'),
             gof_omit = "Log|F",
             add_rows = model_summary_insert_dvnames(named_N_models_base),
             stars = TRUE
)

named_N_models_add <- list(
  "lines+icp" = modelN_lines_icp,
  "+capex" = modelN_lines_icp_capex, 
   "+time"= modelN_lines_icp_time, 
  "+capex+time" = modelN_lines_icp_capex_time)

modelsummary(named_N_models_add,
             output = file.path("Outputs", "table_ms_network_add.docx"),
             title = paste0('Network models.'),
             gof_omit = "Log|F",
             add_rows = model_summary_insert_dvnames(named_N_models_add),
             stars = TRUE
)

# c. Comparison table for Non-network Models - Base ----

stargazer(models_NN_list_base,
          title = paste0("Non-network Models ", years, ". Irregular ID data excluded, without IMOE or CRSE."),
          type = "text",
          column.labels = col_labels_NN_base,
          header=F, #  get rid of r package output text
          single.row = F, # put coefficients and standard errors on same line
          no.space = F, # remove the spaces after each line of coefficients
          #column.sep.width = "3pt", # to reduce column width
          font.size = "small",
          # Manually choose here, if standard errors are vanilla, or assume clusters
          # Needs same nubmer of points in each model being compared though
          omit.stat = c("f", "ser"),
          out = file.path("Outputs", "table_sg_nonntwk_base.html")
)


# d. Comparison table for Non-network Models - Additions ----

stargazer(models_NN_list_add,
          title = paste0("Non-network Model additions ", years, ". Irregular ID data excluded, without IMOE or CRSE."),
          type = "text",
          column.labels = col_labels_NN_add,
          header=F, #  get rid of r package output text
          single.row = F, # put coefficients and standard errors on same line
          no.space = F, # remove the spaces after each line of coefficients
          #column.sep.width = "3pt", # to reduce column width
          font.size = "small",
          # Manually choose here, if standard errors are vanilla, or assume clusters
          # Needs same nubmer of points in each model being compared though
          omit.stat = c("f", "ser"),
          out = file.path("Outputs", "table_sg_nonntwk_adds.html")
)

# Model Summary Non-network models

named_NN_models_base <- list(
  "lines+icp" = modelNN_lines_icp,
  "lines+peakR" = modelNN_lines_peak_rat, 
  "lines+delR" = modelNN_lines_delivery_rat, 
  "lines+del" = modelNN_lines_delivery, 
  "icp+peakR" = modelNN_icp_peak_rat, 
  "icp+delR" = modelNN_icp_delivery_rat, 
  "icp+del" = modelNN_icp_delivery, 
  "+peakR" = modelNN_lines_icp_peak_rat, 
  "+deliveryR" = modelNN_lines_icp_delivery_rat,
  "+delivery" = modelNN_lines_icp_delivery)

modelsummary(named_NN_models_base,
             output = file.path("Outputs", "table_ms_nonnetwork_base.docx"),
             title = paste0('Non-network models.'),
             gof_omit = "Log|F",
             add_rows = model_summary_insert_dvnames(named_NN_models_base),
             stars = TRUE
)

named_NN_models_add <- list(
  "lines+icp" = modelNN_lines_icp,
  "+capex" = modelNN_lines_icp_capex, 
  "+time"= modelNN_lines_icp_time, 
  "+capex+time" = modelNN_lines_icp_capex_time)

modelsummary(named_NN_models_add,
             output = file.path("Outputs", "table_ms_nonnetwork_add.docx"),
             title = paste0('Non-network models.'),
             gof_omit = "Log|F",
             add_rows = model_summary_insert_dvnames(named_NN_models_add),
             stars = TRUE
)

# 3. Comments on Model Selection ----

# Inspection of results shows that substituting either lines or icp with 
# either delivery / peak variables can give models with 3 star coefficients, 
# but in all cases marginally worse AdjRSq for all variations.

# When adding either delivery, deliver_rat or peak_rat to lines + icp, the 
# models are not improved (anova test p values well above 0.05), and the models 
# tend to accommodate the new variable mostly by reducing icp coefficient for 
# network, and lines for non-network models. This suggested the new variable is 
# not explaining data variation not already modeled by icp or lines

# Furthermore, ratcheted variables are harder to project forward for DPP4 period

# We conclude no basis to further pursue inclusion of delivery or peak variables 
# in models. These quantities do fundamentally drive costs, but it simply turns out
# empirically that modeling cost scaling with size is better done with lines and icp

# There _is_ a case for including capex in nonnetwork model:
# (NN opex ~ lines + icp + capex) 
# but not for network opex

# Anova p-value is < 0.05 for NN opex, but not for N opex
# anova(modelN_lines_icp, modelN_lines_icp_capex)
# anova(modelNN_lines_icp, modelNN_lines_icp_capex)

# In addition, the summary statistics indicate that adding capex does indeed 
# improve adjusted R squared, AIC and BIC for NN opex but not for N opex models. 

# For nonnetwork opex, the capex coefficients found here is similar to work by 
# work (CEPA, Frontier). It is associated with a reduction in lines coefficient, 
# suggesting capex is 'doing better' at explaining some of the cost variation 
# previously explained with lines. 

# For NN model fits, the lines coefficient reduces by almost 50% and the icp 
# coefficient by about 25%. We stress that adding capex to this model 
# will not amount to a simple uplift to opex increases, but a change based on 
# the level of capex:

# An edb with a low growth in capex spend would effectively get less opex uplift 
# than it if we use the lines + icp mode without a capex term. 

# Adding capex to the network models results in a small negative coefficient.
# It may be tempting to interpret this as evidence of a 'substitution' effect, with
# a small (4%) reduction in opex spend correlated with increased capex. 

# However, we stress that the standard error in this coefficients is twice the value, 
# indicating that it is not statistically different from zero. And unlike adding capex 
# to the NN opex model, adding capex to the N opex model worsens the fit metrics 
# (adjusted R squared, AIC, BIC).

# Adding a time variable, as suggested by Frontier, indeed improves model fits
# We confirm their conclusion that based on model metrics, the best above models:
# network ~ lines + icp + time
# nonnetwork ~ lines + icp + capex + time
#
# However, for broader non-technical reasons, related to how the opex scale trends
# work is applied as one component of a broader opex allocation process, we have
# decided not to include 'time' terms in our DPP4 draft decision models. 
#
# We have however, provided an 'industry specific differential' uplift to the 
# cost escalations for draft DPP4 opex allowances. 
# See the DPP4 Draft Decisions reasons paper for discussion.


# 4. Iterative model outlier exclusion (IMOE) ----

# In DPP3 we applied an automated, iterative outlier exclusion process. 
# For each model fit, this process would assess 4 goodness of fit metrics for each 
# data point in the data set being fit. If any point failed 3 (or 4) of the 4 tests
# it would be removed and the same model re-fit. This process continued until there
# all points failed at most 2 of these tests. 
#
# We have investigated the impact of included this step or not, and for DPP4 draft
# decisions have included it as a final step in our fitting process.
#
# Applying IMOE improves model fit metrics, by definition. But as we see below, 
# it also has the unexpected consequence for our reference period of input data of 
# excluding data from all years for several EDBs (in network and non-network models). 
#
# In terms of model fits:
#
# Applying IMOE to the preferred network model (icp + lines) results in essentially
# no change, and no statistically significant change in elasticities.
#
# It leads to the exclusion of only four (4) out of 174 points. Refitting with these 
# points excluded results in changes to model coefficients smaller than the standard 
# errors - ie, no statistically significant change in elasticities. 
#
# The root mean squared error RMSE reduces from 0.24 to 0.23, indicated a minor 
# improvement in fit. The adjusted R squared values does not decrease, being constant
# to 2 dp at (0.94). Indeed the -0.004 reduction from 0.942 to 0.93 may be smaller than the 
# standard error in adjusted R square value itself.
#
# AIC and BIC cannot be directly compared for models with different number of data points.
#
# For the non-network model, the exclusion process clearly results in an improved fit 
# with RMSE from 0.29 to 0.22 and adjusted R squared from 0.91 to 0.95,and results in 
# significant changes in the elasticities for icp, lines and capex - with changes larger 
# than the standard errors). 
#
# Inspection (See below) of which points are excluded reveals that the points excluded 
# for the non-network case include ALL data points for two EDBs which lay below the general 
# arc of data points from other EDBs.
#
# As can be verified by changing the reference period for fitting, and re-running the code, 
# applying this comparison to the wider date range (2013-2023) does NOT result in 
# as many data points being excluded (only 5 out of 289). The lower overall noise levels (fewer 
# anomalous data points) in our DPP4 reference period (2018-2023) appears to give 
# rise in some sense to 'tighter model fits' and consequently more data points being 
# excluded when we apply IMOE. 
#
# The case is very strong to support the use of IMOE for non-network models. It is less 
# strong for the network model, with minor improvement in RMSE, effectively no meaningful
# change to adjusted R squared and no statistically significant change in model coefficient 
# and fit parameters.
#
# We have decide to maintain the 2018-2023 reference period, and apply the IMOE process to
# both network and non-network preferred models. This is in line with our general rule to 
# not change from approaches in previous resets without compelling reason. 
#
# An impact assessment (not shown here) of using network elasticities from the model without
# IMOE (while retaining the non-network elasticities with IMOE applied) shows that our
# draft decision to apply IMOE to the network model ultimately results in very minor _increases_
# in opex allowance than if we did not apply IMOE to the network opex model. The difference 
# is of the order of $30k out of $650m pa across all EDBs, and no EDB is worse off with 
# the approach we have taken (compared to calculating network opex elasticities without IMOE) 
# 
# Overall, our draft decision DPP4 elasticities for network opex very similar to final DPP3 
# values, and elasticities for non-network opex which are different from DPP3 due to the 
# inclusion of a capex model term and the specific impact of iterative exclusion of model outliers applied
# to the lower noise data in our reference period 2018-2023.

# As a last step, we calculate std errors using robust errors, clustered on edb
# This does not change the model coefficients, only their standard errors

# network loop
outputs_N_lines_icp_loop_test <- test_outlier_loop1(data_ref, Model = modelN_lines_icp)
modelN_lines_icp_loop_test <- outputs_N_lines_icp_loop_test$model
data_N_lines_icp_loop_test <- outputs_N_lines_icp_loop_test$data_fit

# network robust loop
outputs_N_lines_icp_loop_robust_test <- test_outlier_loop_robust1(data_ref, Model = modelN_lines_icp)
modelN_lines_icp_loop_robust_test <- outputs_N_lines_icp_loop_robust_test$model
data_N_lines_icp_loop_robust_test <- outputs_N_lines_icp_loop_robust_test$data_fit

summary(modelN_lines_icp_loop_test)
summary(modelN_lines_icp_loop_robust_test)

# non-network loop
outputs_NN_lines_icp_loop_test <- test_outlier_loop1(data_ref, Model = modelNN_lines_icp)
modelNN_lines_icp_loop_test <- outputs_NN_lines_icp_loop_test$model
data_NN_lines_icp_loop_test <- outputs_NN_lines_icp_loop_test$data_fit

outputs_NN_lines_icp_capex_loop_test <- test_outlier_loop1(data_ref, Model = modelNN_lines_icp_capex)
modelNN_lines_icp_capex_loop_test <- outputs_NN_lines_icp_capex_loop_test$model
data_NN_lines_icp_capex_loop_test <- outputs_NN_lines_icp_capex_loop_test$data_fit

# non-network robust loop
outputs_NN_lines_icp_capex_loop_robust_test <- test_outlier_loop_robust1(data_ref, Model = modelNN_lines_icp_capex)
modelNN_lines_icp_capex_loop_robust_test <- outputs_NN_lines_icp_capex_loop_robust_test$model
data_NN_lines_icp_capex_loop_robust_test <- outputs_NN_lines_icp_capex_loop_robust_test$data_fit

summary(modelNN_lines_icp_capex_loop_test)
summary(modelNN_lines_icp_capex_loop_robust_test)

# Stargazer tables for final models ---- 

# network
modelN_lines_icp_loop_test$AIC <- AIC(modelN_lines_icp_loop_test)
modelN_lines_icp_loop_test$BIC <- BIC(modelN_lines_icp_loop_test)

# Non-network
modelNN_lines_icp_loop_test$AIC <- AIC(modelNN_lines_icp_loop_test)
modelNN_lines_icp_loop_test$BIC <- BIC(modelNN_lines_icp_loop_test)

# Comparison tables for preferred models with and without IMOE
# Iterative model outlier exclusion

named_models_test <- makeNamedList(modelN_lines_icp,
                                   modelN_lines_icp_loop_test,
                                   modelNN_lines_icp,
                                   modelNN_lines_icp_loop_test,
                                   modelNN_lines_icp_capex,
                                   modelNN_lines_icp_capex_loop_test
                                   )

models_N_list_loop <- list(modelN_lines_icp,
                           modelN_lines_icp_loop_test)

models_NN_list_loop <- list(modelNN_lines_icp,
                            modelNN_lines_icp_loop_test,
                            modelNN_lines_icp_capex,
                            modelNN_lines_icp_capex_loop_test)

colum_labels_N_loop  = c("lines+icp", 
                         "w. IMOE")

colum_labels_NN_loop = c("lines+icp",  
                         "w. IMOE",
                         "lines+icp+capex",  
                         "w. IMOE")

# e. Comparison table for Network Models - iterative model outlier exclusions ----

# Stack N models using stargazer and output to screen
years = paste0(min(data_ref$year), "-", max(data_ref$year))

stargazer(models_N_list_loop,
          title = paste0("Network Model ", years, ". effect of IMOE and CRSE"),
          type = "text",
          column.labels = colum_labels_N_loop,
          header=F, #  get rid of r package output text
          single.row = F, # put coefficients and standard errors on same line
          no.space = F, # remove the spaces after each line of coefficients
          #column.sep.width = "3pt", # to reduce column width
          font.size = "small",
          # Manually choose here, if standard errors are vanilla, or assume clusters
          # Needs same nubmer of points in each model being compared though
          omit.stat = c("f"), # "ser"
          out = file.path("Outputs", "table_sg_network_IMOE.html")
)

# modelsummary table: 
# Compare changes from base, including IMOE, and IMOE + CRSE 
# Iterative Model Outlier Exclusions
# Clustered Robust Standard Errors

named_N_models_loop_robust <- list(
  "Model" = modelN_lines_icp,
  "w. IMOE" = modelN_lines_icp_loop_test,
  "w. IMOE + CRSE" = modelN_lines_icp_loop_robust_test)


modelsummary(named_N_models_loop_robust, 
             #output = file.path("Outputs", "table_ms_network_IMOE_CRSE.docx"),
             title = paste0('Network models, effect of iterative model outlier exclusion and clustered-robust standard errors.'),
             gof_omit = "Log|F",
             add_rows = model_summary_insert_dvnames(named_N_models_loop_robust),
             stars = TRUE
)

# f. Comparison table for Non-network Models - iterative model outlier exclusions ----

stargazer(models_NN_list_loop,
          title = paste0("Non-network Model ", years, ". effect of IMOE"),
          type = "text",
          column.labels = colum_labels_NN_loop,
          header=F, #  get rid of r package output text
          single.row = F, # put coefficients and standard errors on same line
          no.space = F, # remove the spaces after each line of coefficients
          #column.sep.width = "3pt", # to reduce column width
          font.size = "small",
          # Manually choose here, if standard errors are vanilla, or assume clusters
          # Needs same number of points in each model being compared though
          omit.stat = c("f"), # "ser"
          out = file.path("Outputs", "table_sg_nonnetwork_IMOE.html")
)

# modelsummary table: 
# Compare changes from base, including IMOE, and IMOE + CRSE 
# Iterative Model Outlier Exclusions
# Clustered Robust Standard Errors

named_NN_models_loop_robust <- list(
  "Model" = modelNN_lines_icp_capex,
  "w. IMOE" = modelNN_lines_icp_capex_loop_test,
  "w. IMOE + CRSE" = modelNN_lines_icp_capex_loop_robust_test)

modelsummary(named_NN_models_loop_robust, 
             output = file.path("Outputs", "table_ms_nonnetwork_IMOE_CRSE.docx"),
             title = paste0('Non-network models, effect of iterative model outlier exclusion and clustered-robust standard errors.'),
             gof_omit = "Log|F",
             add_rows = model_summary_insert_dvnames(named_NN_models_loop_robust),
             stars = TRUE
             )

# g. Final network and non-network models and elasticities ----

# stargazer will not work nicely with lm_robust() objects
# Alternative is to force in the se(clusters) into the lm() equivalent object 
# or use eg. modelsummary to make the tables

named_models_final_robust <- list("Network" = modelN_lines_icp_loop_robust_test,
                                  "Non-network" = modelNN_lines_icp_capex_loop_robust_test)

modelsummary(named_models_final_robust, 
             output = file.path("Outputs", "table_ms_final_models.docx"),
             gof_omit = "Log|F",
             #add_rows = model_summary_insert_dvnames(named_models_final_robust),
             stars = TRUE,
             title = paste0('Final models, with robust standard errors (clustered by EDB), 
                            iterative model outlier exclusion, and irregular input data excluded.')
)


# 5 Charting examination of points removed by IMOE process ----

# Below shows which points are excluded by the IMOE process, and therefore helps understand
# how this leads to a change in coefficients for NN opex models, but not N opex models.

# Plots of Network Outliers 
var_rank = "icp"
data_plotN <- copy(data_N_lines_icp_loop_test) %>% 
  group_by(edb) %>% 
  mutate(var.mean = max(get(var_rank))) %>% 
  ungroup %>% 
  arrange(desc(var.mean)) %>% 
  mutate(edb = factor(edb, levels=unique(edb))) %>%
  arrange(edb, year) %>% 
  mutate(year_fct = as.factor(year))
data_plotN <- data.table(data_plotN)

p_N <- ggplot(data_plotN, aes(x = get(var_rank), y = network, alpha = year, color = edb)) + 
  geom_point(size = 4)+
  # Highlight the Outlier ones
  geom_point(data=subset(data_plotN,N1outlier=="1"), shape = 21, fill = "black", alpha = 1, size = 4) + 
  theme_light() +
  theme(axis.text.x = element_text(angle = 90, vjust = 0.5, hjust=1))+
  theme(legend.position = "none") +
  labs(y = "log Actual Network Cost[$])",
       x = paste0("log(",var_rank, ")"),
       title = paste0("Network Costs vs. ", var_rank, " ( ", year_min_fit, " - ", year_max_fit, " )"),
       subtitle = "Coloured by edb, shading by year. Black points excluded by iterative outlier method") +
  facet_wrap(~edb, scales = "free")

grid.arrange(p_N)

p_N_icp <- ggplot(data_plotN, aes(x = icp, y = network, alpha = year, color = edb)) + 
  geom_point(size = 4)+
  # Highlight the OUtlier ones
  geom_point(data=subset(data_plotN,N1outlier=="1"), shape = 21, fill = "black", alpha = 1, size = 4) + 
  theme_light() +
  theme(axis.text.x = element_text(angle = 90, vjust = 0.5, hjust=1))+
  theme(legend.position = "none") +
  labs(y = "log Actual Network Cost[$])",
       x = paste0("log(icp)"),
       title = paste0("Network Costs vs. ", var_rank, " ( ", year_min_fit, " - ", year_max_fit, " )"),
       subtitle = "Coloured by edb, shading by year. Black points excluded by iterative outlier method") 

p_N_lines <- ggplot(data_plotN, aes(x = lines, y = network, alpha = year, color = edb)) + 
  geom_point(size = 4)+
  # Highlight the OUtlier ones
  geom_point(data=subset(data_plotN,N1outlier=="1"), shape = 21, fill = "black", alpha = 1, size = 4) + 
  theme_light() +
  theme(axis.text.x = element_text(angle = 90, vjust = 0.5, hjust=1))+
  theme(legend.position = "none") +
  labs(y = "log Actual Network Cost[$])",
       x = paste0("log(lines)"),
       title = paste0("Network Costs vs. ", var_rank, " ( ", year_min_fit, " - ", year_max_fit, " )"),
       subtitle = "Coloured by edb, shading by year. Black points excluded by iterative outlier method") 

grid.arrange(p_N_icp,
            p_N_lines,
            ncol =1)

# Plots of Non-network Outliers 

var_rank = "lines"
data_plotNN <- copy(data_NN_lines_icp_capex_loop_test) %>% 
  group_by(edb) %>% 
  mutate(var.mean = max(get(var_rank))) %>% 
  ungroup %>% 
  arrange(desc(var.mean)) %>% 
  mutate(edb = factor(edb, levels=unique(edb))) %>%
  arrange(edb, year) %>% 
  mutate(year_fct = as.factor(year))
data_plotNN <- data.table(data_plotNN)

p_NN <- ggplot(data_plotNN, aes(x = get(var_rank), y = nonnetwork, alpha = year, color = edb)) + 
  geom_point(size = 4)+
  # Highlight the OUtlier ones
  geom_point(data=subset(data_plotNN,N1outlier=="1"), shape = 21, fill = "black", alpha = 1, size = 4) + 
  theme_light() +
  theme(axis.text.x = element_text(angle = 90, vjust = 0.5, hjust=1))+
  theme(legend.position = "none") +
  labs(y = "log Actual Network Cost[$])",
       x = paste0("log(",var_rank, ")"),
       title = paste0("Non-network Costs vs. ", var_rank, " ( ", year_min_fit, " - ", year_max_fit, " )"),
       subtitle = "Coloured by edb, shading by year. Black points excluded by iterative outlier method") +
  facet_wrap(~edb, scales = "free")

grid.arrange(p_NN)

p_NN_icp <- ggplot(data_plotNN, aes(x = icp, y = nonnetwork, alpha = year, color = edb)) + 
  geom_point(size = 4)+
  # Highlight the Outlier ones
  geom_point(data=subset(data_plotNN,N1outlier=="1"), shape = 21, fill = "black", alpha = 1, size = 4) + 
  theme_light() +
  theme(axis.text.x = element_text(angle = 90, vjust = 0.5, hjust=1))+
  theme(legend.position = "none") +
  labs(y = "log Actual Non-network Cost[$])",
       x = "log(icp)",
       title = paste0("Non-network Costs vs. ", var_rank, " ( ", year_min_fit, " - ", year_max_fit, " )"),
       subtitle = "Coloured by edb, shading by year. Black points excluded by iterative outlier method") 

p_NN_lines <- ggplot(data_plotNN, aes(x = lines, y = nonnetwork, alpha = year, color = edb)) + 
  geom_point(size = 4)+
  # Highlight the OUtlier ones
  geom_point(data=subset(data_plotNN,N1outlier=="1"), shape = 21, fill = "black", alpha = 1, size = 4) + 
  theme_light() +
  theme(axis.text.x = element_text(angle = 90, vjust = 0.5, hjust=1))+
  theme(legend.position = "none") +
  labs(y = "log Actual Non-network Cost[$])",
       x = "log(lines)",
       title = paste0("Non-network Costs vs. ", var_rank, " ( ", year_min_fit, " - ", year_max_fit, " )"),
       subtitle = "Coloured by edb, shading by year. Black points excluded by iterative outlier method") 

grid.arrange(p_NN_icp, 
            p_NN_lines,
            ncol = 1)

# Excluded data points
excluded_N <- data_plotN[N1outlier=="1", c("year", "edb")][ , "opex" := "network"] 
excluded_NN <- data_plotNN[N1outlier=="1", c("year", "edb")][ , "opex" := "nonnetwork"] 
excluded <- rbind(excluded_N, excluded_NN)

#Excluded data (edb-year)
nrow(excluded_NN)
uniqueN(excluded_NN$edb)
excluded_NN

